// Part of the constructs library.

// Builds parts for a low-power switch. The switch is double-terminal and
// can be either single pole, or double pole. Based on do_build variable
// below, it either lays out the parts for printing, or previews them as abn
// assembled switch. Modules are:

// switch_arm( arm_w ) - makes the arm of the switch with arm_w thickness 
// switch_housing( arm_w, contact_w, is_double = 1) - makes switch housing.
// Adjust contact_w to control the amount of space inserhted on each side of
// the switch arm. If is_double is set to 0, a spdt housing will be build,
// with space for the contact on only one side.

// By Nenad Rijavec
// Feel free to use and modify as you see fit.


use <plib/cylinder_wedge.scad>

do_build = 1 ; // set to 0 to preview assembled switch
is_double = 0 ; // 1 - dpdt, 0- spdt
bld_arm = 1 ; // if building, build the arm
bld_housing = 1 ; // if building, build the housing
box_w = 11.2 ;
box_l = 24 ;
box_wall = 2 ; // housing wall thickness in mm
contact_w = 1.0 ; // contact width - the space (mm) between arm and housing
space_w = box_w - 2 * box_wall ;
//space_w = arm_w + (1+is_double)*contact_w + 0.2 ;
space_l = box_l - 2*box_wall ;
// 0.2 below is extra space to account for print growth of parts
// space is for a single side, will be doubled for dpdt
arm_w = space_w - 0.2 - (1+is_double)*contact_w ;
//echo( arm_w, contact_w, space_w, space_l, box_w, box_l ) ;

module switch_arm()
{
	difference() {
		union() {
			cylinder( arm_w, d=6, $fn=100 ) ;
			translate( [ 0, -2.5, 0 ] )
				cube( [ 17.5, 5, arm_w ] ) ;
			translate( [ 17.5, 0, 0 ] )
				cylinder( arm_w, d=5, $fn=100 ) ;
			intersection() {
				hull() {
				translate( [ -3, 0, 0 ] )
				cylinder_wedge( 40, arm_w+5, -30, 60 )
					cylinder( arm_w, d=25, $fn=100 ) ;
				translate( [ -5, 0, 0 ] )
				cylinder_wedge( 40, arm_w+5, -30, 60 )
					cylinder( arm_w, d=25, $fn=100 ) ;
				}
				translate( [ -2, -100, -1 ] )
					cube( [ 100, 200, arm_w + 2 ] ) ;
			}

		}
		union() {
			// axle
			translate( [ 0, 0, -1 ] )
				cylinder( arm_w+2, d=2, $fn=100 ) ;
			// spring passthrough
			translate( [ 6, 0, -1 ] )
				cylinder( arm_w+2, d=2, $fn=100 ) ;
			translate( [ 8.7, -5, arm_w+0.7 ] )
				rotate( [ 25, 0, 0 ] ) 
				cube( [ 9.2, 5, 3 ],center=true ) ;
			translate( [ 8.7, 5, arm_w+0.7 ] )
				rotate( [ -25, 0, 0 ] ) 
				cube( [ 9.2, 5, 3 ],center=true ) ;
			if ( is_double ) {
				translate( [ 8.7, -5, -0.7 ] )
					rotate( [ -25, 0, 0 ] ) 
					cube( [ 9.2, 5, 3 ],center=true ) ;
				translate( [ 8.7, 5, -0.7 ] )
					rotate( [ 25, 0, 0 ] ) 
					cube( [ 9.2, 5, 3 ],center=true ) ;
			}
		}
	}
	
}

module switch_housing( )
{ 
	box_center = box_l/2 ;
	difference() {
		union() {
			cube( [ box_w, box_l, 15 ] ) ;
		}
		union() {
			translate( [ box_wall, box_wall, 1, ] )
				cube( [ space_w, space_l, 20 ] ) ;
			translate( [ -1, box_center, 6 ] ) 
			rotate( [ 0, 90, 0 ] )
				cylinder( box_w+2, d=2, $fn=100 ) ;
			translate( [ -1, box_center-1, -1 ] ) 
				cube( [ box_w+2, 2, 3 ] ) ;
			translate( [ 0, box_center, 0 ] ) 
				cylinder( 6, d=2, $fn=100 ) ;
			translate( [ box_w, box_center, 0 ] ) 
				cylinder( 6, d=2, $fn=100 ) ;
			translate( [ -1, box_center, 6 ] ) 
			rotate( [ 0, 90, 0 ] )
			cylinder_wedge( 20, box_w+5, 133, 90 )
			difference() {
				cylinder( box_w+2, d=15, $fn=100 ) ;
				translate( [ 0,0,-1])
				cylinder( box_w+4, d=10, $fn=100 ) ;
			}
			// terminal
			translate( [ box_wall+1, box_wall, -1, ] )
				cylinder( 12, d=2, $fn=100 ) ;
			// terminal
			translate( [ box_wall+1, box_wall+space_l, -1, ] )
				cylinder( 12, d=2, $fn=100 ) ;
			// pole vertical
			translate( [ box_wall, box_l/2-4, -1, ] )
				cylinder( 10, d=2, $fn=100 ) ;
			// pole horizontal
			translate( [ box_wall, box_l/2-4, 8, ] )
			rotate( [ -90, 0, 0 ] )
				cylinder( 9.5, d=2, $fn=100 ) ;
			if ( is_double ) {
				translate([box_wall+space_w-1, box_wall, -1,])
					cylinder( 10, d=2, $fn=100 ) ;
				translate([box_wall+space_w, box_l/2-4, -1,])
					cylinder( 10, d=2, $fn=100 ) ;
				translate( [ box_wall+space_w, box_l/2-4, 8, ] )
				rotate( [ -90, 0, 0 ] )
					cylinder( 8.5, d=2, $fn=100 ) ;
				translate([box_wall+space_w-1,
						box_wall+space_l, -1, ] )
					cylinder( 10, d=2, $fn=100 ) ;
			}
		}
	}
}

module switch_holder( arm_w, contact_w, box_wall, is_double = 1)
{ 
	
	// 0.2 below is extra space to account for print growth of parts
	space_w = arm_w + (1+is_double)*contact_w + 0.2 ;
	box_w = space_w + 2*box_wall+4 ;
	box_l = 24+5 ;
	space_l = box_l - 2*box_wall ;
	box_center = box_l/2 ;
	difference() {
		cube( [ box_w, box_l, 19 ] ) ;
		union() {
			translate( [ 2.5, -1, 1.5 ] ) 
				cube( [ box_w-5, box_l+2, 19 ] ) ;
			translate( [ 1.5, 1.5, 1.5 ] ) 
				cube( [ box_w-3, box_l-3, 19 ] ) ;
			translate( [ -1, 2.5, 1.5 ] ) 
				cube( [ box_w+2, box_l-5, 19 ] ) ;
			translate( [ 2.5, 2.5, -1 ] ) 
				cube( [ box_w-5, box_l-5, 19+2 ] ) ;
		}
	}
}

if ( do_build ) {
	if ( bld_arm )
		if ( is_double ) {
			translate( [ 0, 0, 3.0 ] )
				rotate( [ 0, -90, 0 ] )
				switch_arm() ;
			//build base
			translate( [ -8-arm_w/2, -8, 0 ] )
				cube( [ 16, 16, 0.3 ] ) ;
		} else
			switch_arm() ;
	if ( bld_housing ) 
		translate( [ 0, 10*bld_arm, 0 ] )
			switch_housing() ; 
} else {
//	space_w = arm_w + (1+is_double)*contact_w + 0.2 ;
//	box_w = space_w + 2*box_wall ;
//	box_l = 24 ;
	// uncomment intersection below to see the cutaway view
	intersection() { union() {
	rotate( [ 37, 0, 0 ] )
		translate( [ arm_w/2, 0, 0 ] ) 
		rotate( [ 0, -90, 0 ] )
			switch_arm( arm_w) ;
	translate( [ -box_w/2, -box_l/2, -6 ] ) 
		switch_housing( arm_w, contact_w, is_double) ;
	}
	translate( [ -40, -40, -40 ] ) cube( [ 40, 100, 100 ] ) ;
	}

	translate( [ -box_w/2-2, -box_l/2-2, -6-2 ] ) 
	switch_holder( arm_w, contact_w, box_wall, is_double) ;
}


// nenad me fecit
