// Part of the openscad_primitives library

// Operator makehollow takes an object and makes it hollow with the walls of
// specified thickness. The object is the first child in the scope.

// Call syntax:
// makehollow( wall, d=0 )
// where:
//	wall - wall thickness in mm
//      d - optional filleting diameter. If nonzero, the outer shell will be
//	filleted via minkowski with the sphere of a given diameter. Note
//	that this will increase the wall thickness by d/2. This option is
//	given so the filleted object would not be imaged over and over
//	again to obtain the inner cavity, leading to long rendering times.

// Actual wall thickness will only approximate the desired thickness.

// There are no assumptions on the shape of the object to be hollowed, but
// it will be constructed four times during the operation, so the code may
// be very slow if the object is complex.

// Complex objects may not display properly in preview. Use render (as in
// the example below) or F6 to generate and view the mesh.

// By Nenad Rijavec.
// Feel free to use and modify as you see fit.


// Algorithm:
// Expand the object by a small margin in all three dimensions, by applying
// a minkowski with a very small cube, centered at origin. Subtract the
// original object to get a thin shell. Grow the thin shell to twice the
// desired wall thickness via another minkowski. Note that the grow cube
// dimension is decreased by the thin_w that was applied earlier. Subtract
// this shell from the original object. What remains is the shape of the
// cavity that can then be subtracted from the original object to get the
// desired shell.

module makehollow( wall, d=0 )
{
	thin_w = 0.1 ;
	w2 = 2 * wall - thin_w ; 
	difference() {
		if ( d== 0 )
			children(0) ; // object to be hollowed
		else
			minkowski() {
				children(0) ; sphere( d=d, $fn=100 ) ;
		}
		difference() { // cavity to be removed
			children(0) ;
			minkowski() { // object volume outside of the cavity
				difference() { // thin shell
					minkowski() { 
						children(0) ;
						cube([thin_w, thin_w, thin_w],
							center = true ) ;
					}
					children(0) ;
				}
				cube( [ w2, w2, w2 ], center=true ) ;
			}
		}
	}
}


points = [ [ 0,0], [5,-10], [ -1, -20 ], [ 11, -20], [6,-10 ], [10,0], [30,0], [30,20], [0,20] ] ;

intersection() {
render(2) makehollow( 2, 5 ) { linear_extrude( 10 ) polygon( points ) ; }
translate( [ -30, -30, -5 ] ) cube( [ 200, 200, 9 ] ) ;
}
